;
;Scratchpad
;Author:		Desi Quintans <me@desiquintans.com>
;Website:		http://www.desiquintans.com/scratchpad
;
;Script Function:
;	Scratchpad is a quick-and-dirty note-taking tool with a minimum of frills, meant to show
;	up when you need it and get out of the way when you don't. Programmers can use it to
;	document code changes as they happen without having an extra program on their taskbar, or
;	you could use it to store snippets of text temporarily.
;

#NoEnv
#Persistent
#SingleInstance, Force
#MaxHotkeysPerInterval, 200
SendMode Input
FileEncoding UTF-8
SetWorkingDir %A_ScriptDir%
OnExit, SaveOnExit

Gui +LastFound
GUI1 := WinExist()

Menu,Tray, NoStandard
Menu, Tray, Add, Show/Hide, GuiShowHide
Menu, Tray, Add
Menu, Tray, Default, Show/Hide
Menu, Tray, Click, 1
Menu, Tray, Standard

LastActiveFile =
RecordedLastModified =
DefaultWindowTitle = Scratchpad (desiquintans.com/scratchpad)
UnsavedWindowTitle = Scratchpad * (desiquintans.com/scratchpad)

OnMessage(0x112,"WM_SYSCOMMAND")

Gui,+AlwaysOnTop

if 0 > 0 ;If a command-line parameter (%0%) has been passed to the exe
{
	if 1 = "/min"
	{
		Gui, Show, Hide h582 w466, %DefaultWindowTitle%
	}
}
else
{
	IfExist, ScratchpadData.ini
	{
		IniRead, minimisedStatus, ScratchpadData.ini, Settings, StartMinimised
		
		if minimisedStatus = 1
		{
			Gui, Show, Hide h582 w466, %DefaultWindowTitle%
		}
		else
		{
			Gui, Show, h582 w466, %DefaultWindowTitle%
		}
	}
	else
	{
		Gui, Show, h582 w466, %DefaultWindowTitle%
	}
}

	; All this stuff is up here so that it's not affected by the font change. Easier
	; to do it like this than to change the font again.
	Gui, Add, GroupBox, x22 y445 w420 h75 , 
		Gui, Add, Text, x32 y463 w60 h20 , Current file:
		Gui, Add, Edit, x92 y460 w340 h20 vHistory ReadOnly -Background c808080, Unknown file
		Gui, Add, CheckBox, x32 y490 w130 h20 checked vSaveOnExit gUpdateVars, Autosave on exit
		Gui, Add, CheckBox, x162 y490 w130 h20 vStartMinimised gUpdateVars, Start minimised
		Gui, Add, Button, x310 y487 w122 h26 gNew, New From Template
	
	Gui, Add, GroupBox, x82 y524 w290 h40 , 
		Gui, Add, Text, x92 y540 w100 h20 , Show/Hide Hotkey:
		Gui, Add, Hotkey, x205 y537 w110 h20 vVisibilityHotkey gSetBind, s
		Gui, Add, CheckBox, x322 y532 w40 h30 checked vUseWinkey gUpdateVars, Win

	Gui, Add, Button, x444 y560 w20 h20 gHelp, ?
		
	gui, font, s10, Consolas
	Gui, Color, B0C4DE
	Gui, Add, Edit, x4 y4 w458 h440 WantTab T8 vText gUpdateText
	GuiControl, Focus, Text

	GoSub, ReadFromIni
	GoSub, SetBind
	GoSub, LoadOnStart
	GoSub, UpdateVars
Return

Help:
WinSet, AlwaysOnTop, Off
MsgBox, 32, Help  Scratchpad,
(
Ctrl+S		Save
Ctrl+Shift+S	Save As
Ctrl+O		Open
Ctrl+N		New file from template
		(template/template.txt)

Escape		Hide Scratchpad (if it's active)
Show/Hide key	Toggle Scratchpad's visibility

Note: Autosaving creates a backup in the same folder as the current file.
Scratchpad also refreshes the active file (if necessary) whenever you unhide it.

You are using Scratchpad v7, released 29 June 2013.

desiquintans.com/scratchpad
)
WinSet, AlwaysOnTop, On
return

#IfWinActive, Scratchpad ahk_class AutoHotkeyGUI
	Escape::Gosub, GuiHideOnly
	+^s::GoSub, SaveAs
	^s::GoSub, Save
	^o::GoSub, Open
	^n::GoSub, New
	Pause::
		WinSet, AlwaysOnTop, Off
		GuiControlGet,dropdown,,History
		
		MsgBox, LastActiveFile: %LastActiveFile%`nHistory: %dropdown%
		WinSet, AlwaysOnTop, On
	return

#IfWinActive ; Undoes the context-sensitivity from above.

SetBind:
	Gui, Submit, NoHide

	if UseWinKey = 1
		AccessKey := "#" VisibilityHotkey
	else
		AccessKey = VisibilityHotkey
	
	if WinkeyStatus_old = 1
	{
		AccessKey_old := "#" AccessKey_old
		Hotkey,%AccessKey_old%, GuiShowHide, off UseErrorLevel
	}
	else
	{
		Hotkey,%AccessKey_old%, GuiShowHide, off UseErrorLevel
	}
	
	Hotkey,%AccessKey%, GuiShowHide, on UseErrorLevel
	
	AccessKey_old := VisibilityHotkey
	WinkeyStatus_old := UseWinkey
	
	GoSub, UpdateVars
	GuiControl, Focus, Text
Return

WriteToIni:
	IfExist, ScratchpadData.ini
		FileDelete, ScratchpadData.ini

	IniWrite, %SaveOnExit%, ScratchpadData.ini, Settings, SaveOnExit
	IniWrite, %StartMinimised%, ScratchpadData.ini, Settings, StartMinimised
	IniWrite, %VisibilityHotkey%, ScratchpadData.ini, Settings, VisibilityHotkey
	IniWrite, %UseWinkey%, ScratchpadData.ini, Settings, UseWinkey
	IniWrite, %LastActiveFile%, ScratchpadData.ini, Settings, LastActiveFile
	
	WinGetPos, WinX, WinY,,,Scratchpad (desiquintans.com/scratchpad)
	IniWrite, %WinX%, ScratchpadData.ini, Settings, WindowPositionX
	IniWrite, %WinY%, ScratchpadData.ini, Settings, WindowPositionY
	
return

ReadFromIni:
	IfExist, ScratchpadData.ini
	{
		IniRead, newSaveOnExit, ScratchpadData.ini, Settings, SaveOnExit
		IniRead, newStartMinimised, ScratchpadData.ini, Settings, StartMinimised
		IniRead, newVisibilityHotkey, ScratchpadData.ini, Settings, VisibilityHotkey
		IniRead, newUseWinkey, ScratchpadData.ini, Settings, UseWinkey
		IniRead, LastActiveFile, ScratchpadData.ini, Settings, LastActiveFile
		IniRead, WinX, ScratchpadData.ini, Settings, WindowPositionX
		IniRead, WinY, ScratchpadData.ini, Settings, WindowPositionY
		
		GuiControl,, SaveOnExit, %newSaveOnExit%
		GuiControl,, StartMinimised, %newStartMinimised%
		GuiControl,, VisibilityHotkey, %newVisibilityHotkey%
		GuiControl,, UseWinkey, %newUseWinkey%
		GuiControl,, History, %LastActiveFile%
		
		WinMove, WinX, WinY
	}
return

LoadOnStart:
	if LastActiveFile ; Not blank
	{
		if FileExist(LastActiveFile)
		{
			FileRead, filecontents, %LastActiveFile%
			GuiControl,,Text, %filecontents%
			FileGetTime, RecordedLastModified, %LastActiveFile%, M
		}
		else
		{
			WinSet, AlwaysOnTop, Off
			MsgBox, 16, File not found  Scratchpad, The last used file could not be found. Has it been moved or deleted?`n`nLoading template instead.`n`n(%LastActiveFile%)
			WinSet, AlwaysOnTop, On
			
			GoSub, LoadTemplate
		}

		GoSub, UpdateVars
	}
	else
	{
		GoSub, LoadTemplate
	}
return

Open:
	WinSet, AlwaysOnTop, Off
	FileSelectFile, openfilename,3,,Open a file?  Scratchpad, Documents (*.txt)
	
	if ErrorLevel
		return
	
	if openfilename != ""
	{
		FileRead, filecontents, %openfilename%
		GuiControl,,Text, %filecontents%
		LastActiveFile := openfilename
		GuiControl,,History, %LastActiveFile%
		FileGetTime, RecordedLastModified, %LastActiveFile%, M
		
		GoSub, UpdateVars
	}
	
	WinSet, AlwaysOnTop, On
return

New:
	GoSub, Save
	GoSub, LoadTemplate
	GoSub, UpdateVars
return

UpdateVars:
		Gui, Submit, NoHide
		GoSub, WriteToIni
return

UpdateText:
	GoSub, UpdateVars
	WinSetTitle, %UnsavedWindowTitle%
return

WM_SYSCOMMAND(wParam)
{
  if (A_Gui && wParam = 0xF020) ; SC_MINIMIZE
  {
    Gui, Hide
    return 0
  }
}

GuiShowHide:
	If DllCall( "IsWindowVisible", "UInt",GUI1 )
	{
		Gui, Hide
	}
	else
	{
  		RefreshActiveFile(LastActiveFile)
  		Gui, Show
	}
Return

GuiHideOnly:
	Gui, Hide
return

LoadTemplate:
	filecontents = ""
	IfExist, template/template.txt
		FileRead, filecontents, template/template.txt
	GuiControl,,Text, %filecontents%
	
	LastActiveFile =
	GuiControl,,History, Editing a new template
return

SaveAs:
	WinSet, AlwaysOnTop, Off
	
	if LastActiveFile ;is not blank
		filename := LastActiveFile
	else
		filename = %A_WorkingDir%\%A_DD% %A_MMM% %A_YYYY% %A_Hour%%A_Min%.txt
			
	FileSelectFile, savefilename, S24, %filename%,Save this file?  Scratchpad, Documents (*.txt)
	
	if ErrorLevel = 0
	{
		if savefilename != ""
		{
			if RegExMatch(savefilename, "(.txt)$")
			{
				FileDelete, %savefilename%
				FileAppend, %Text%, %savefilename%
				LastActiveFile := savefilename
			}
			else
			{
				FileDelete, %savefilename%.txt
				FileAppend, %Text%, %savefilename%.txt
				LastActiveFile := savefilename . ".txt"
			}
			
			GuiControl,,History, %LastActiveFile%
			GoSub, UpdateVars
			WinSetTitle, %DefaultWindowTitle%
			FileGetTime, RecordedLastModified, %LastActiveFile%, M
		}
	}
	
	WinSet, AlwaysOnTop, On
return

Save:
	WinSet, AlwaysOnTop, Off
	
	if LastActiveFile ;not blank
	{
		if FileExist(LastActiveFile)
		{
			if ConfirmSave(LastActiveFile) = 1
			{
				FileDelete, %LastActiveFile%
				FileAppend, %Text%, %LastActiveFile%
				
				GuiControl,,History, %LastActiveFile%
				GoSub, UpdateVars
				WinSetTitle, %DefaultWindowTitle%
				
				FileGetTime, RecordedLastModified, %LastActiveFile%, M
			}
		}
		else
		{
			GoSub, SaveAs
		}
	}
	else
	{
		GoSub, SaveAs
	}
		
	WinSet, AlwaysOnTop, On
return

SaveOnExit:
	GoSub, UpdateVars
	
	WinSet, AlwaysOnTop, Off
	
	if SaveOnExit = 1
	{
		if LastActiveFile ;not blank
		{
			if FileExist(LastActiveFile)
			{
				if ConfirmSave(LastActiveFile) = 1
				{
					BackupFilename := LastActiveFile . ".bak"
					FileCopy, %LastActiveFile%, %BackupFilename%, 1
					
					FileDelete, %LastActiveFile%
					FileAppend, %Text%, %LastActiveFile%
				}
			}
			else
			{
				GoSub, SaveAs
			}
		}
		else
		{
			GoSub, SaveAs
		}
	}
	
	ExitApp
return

CheckTimestamp(File)
{
	global RecordedLastModified
	FileGetTime, CurrentLastModified, %File%, M
	EnvSub, CurrentLastModified, %RecordedLastModified%, Seconds
	
	if CurrentLastModified > 0
	{
		return 1
	}
	else
	{
		return 0
	}
}

ConfirmSave(File)
{
	if CheckTimestamp(File) = 1
	{
		SplitPath, File, Filename
		MsgBox, 52, This file has been changed since you last opened it  Scratchpad, Your active file, '%Filename%' was changed after it was opened in Scratchpad. Are you sure you want to save Scratchpad's version?
		
		IfMsgBox Yes
		{
			return 1
		}
		else
		{
			return 0
		}
	}
	else
	{
		return 1
	}
}

RefreshActiveFile(File)
{
	if CheckTimestamp(File) = 1
	{
		GoSub, LoadOnStart
	}
}